from . import SoftChoiceCharField

try:
    import babel

    currency_names = babel.Locale('en').currencies
except ImportError:  # pragma: no cover
    currency_names = {}


class CurrencyField(SoftChoiceCharField):
    # TODO: add locale support to .formfield()
    def __init__(self, **kwargs):
        self.currencies = kwargs.pop('currencies', ('EUR',))
        kwargs.setdefault('max_length', 3)
        super(CurrencyField, self).__init__(**kwargs)

    def construct_choices(self):
        return [(code, currency_names.get(code, code)) for code in self.currencies]



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from . import SoftChoiceCharField

try:
    import babel

    language_names = babel.Locale('en').languages
except ImportError:  # pragma: no cover
    language_names = dict(settings.LANGUAGES)


class LanguageField(SoftChoiceCharField):
    # TODO: add locale support to .formfield()
    def __init__(self, **kwargs):
        kwargs.setdefault('max_length', 10)
        self.languages = kwargs.pop('languages', settings.LANGUAGES)
        if not all(len(code) <= kwargs['max_length'] for code in self.languages):
            raise ImproperlyConfigured(
                'not all languages fit in max_length=%d characters' % kwargs['max_length']
            )
        super(LanguageField, self).__init__(**kwargs)

    def construct_choices(self):
        return [(code, language_names.get(code, code)) for code in self.languages]



from django.core.exceptions import ImproperlyConfigured

from . import SoftChoiceCharField


class TimezoneField(SoftChoiceCharField):
    # TODO: add locale support to .formfield()
    def __init__(self, **kwargs):
        if 'timezones' not in kwargs:
            try:
                import pytz
                kwargs['timezones'] = pytz.common_timezones
            except ImportError:  # pragma: no cover
                raise ImproperlyConfigured('either declare `timezones` for %r or install `pytz`' % self)
        self.timezones = kwargs.pop('timezones', ())
        kwargs.setdefault('max_length', 65)
        super(TimezoneField, self).__init__(**kwargs)

    def construct_choices(self):
        return [(tz, tz.replace("_", " ")) for tz in self.timezones]



from django.db import models


class SoftChoiceMixin(object):
    def __init__(self, **kwargs):
        if callable(kwargs.get('choices')):
            self.construct_choices = kwargs['choices']
        kwargs['choices'] = self.construct_choices()
        super(SoftChoiceMixin, self).__init__(**kwargs)

    def construct_choices(self):  # pragma: no cover
        return []

    def deconstruct(self):  # pragma: no cover
        # only run when creating migrations, so no-cover
        name, path, args, kwargs = super(SoftChoiceMixin, self).deconstruct()
        kwargs.pop('choices', None)
        return (name, path, args, kwargs)


class SoftChoiceCharField(SoftChoiceMixin, models.CharField):
    pass



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(
    name='django-soft-choice-fields',
    version='0.1.0',
    author='Aarni Koskela',
    author_email='akx@iki.fi',
    packages=find_packages('.', include=('softchoice*')),
    include_package_data=True,
    license='MIT',
)



import pytest
from django.core.exceptions import ImproperlyConfigured
from django.db.migrations.autodetector import MigrationAutodetector
from django.db.migrations.loader import MigrationLoader
from django.db.migrations.state import ProjectState
from django.apps import apps
from django.db.migrations.writer import MigrationWriter
from django.forms.fields import ChoiceField

from softchoice.fields.language import LanguageField
from tests.models import Model


def test_migration():
    """
    Test that migrations for soft choice fields don't include the `choices=` kwarg.
    """
    loader = MigrationLoader(None, ignore_no_migrations=True)
    autodetector = MigrationAutodetector(
        loader.project_state(),
        ProjectState.from_apps(apps),
    )
    changes = autodetector.changes(
        graph=loader.graph,
    )
    writer = MigrationWriter(changes['tests'][0])
    migration_string = writer.as_string()
    assert b'choices=' not in migration_string


@pytest.mark.parametrize('field', [
    'currency',
    'language',
    'timezone',
])
def test_field(field):
    m = Model()
    formfield = m._meta.get_field(field).formfield()
    assert isinstance(formfield, ChoiceField)
    assert len(formfield.choices)


def test_language_validation():
    with pytest.raises(ImproperlyConfigured):
        LanguageField(languages=['neenerneenerneener'])



from django.db import models

from softchoice.fields import SoftChoiceCharField
from softchoice.fields.currency import CurrencyField
from softchoice.fields.language import LanguageField
from softchoice.fields.timezone import TimezoneField


def generate_choices():
    for x in range(10):
        yield [str(x), 'oo' * x]


class Model(models.Model):
    currency = CurrencyField(currencies=['EUR', 'USD', 'JPY'])
    language = LanguageField(languages=['en', 'fi', 'sv'])
    timezone = TimezoneField()
    custom = SoftChoiceCharField(max_length=20, choices=generate_choices)
